# -*- coding: utf-8 -*-

from functools import partial

from .base import BaseRequest
from .base import BaseResponse
from ..element_maker import E_soap_env
from ..element_maker import E_ns1
from ..element_maker import E_ns
from ..namespaces import ns
from ..namespaces import ns1
from ..security.sign import EPOVOpenSSLSigner
from .models import RequestTypes


class GetIncomingQueueStatisticsRequest(BaseRequest):
    u"""Запрос метода GetIncomingQueueStatisticsRequest.

    Метод используется для опроса состояния очереди в системе СМЭВ3.
    """

    # Шаблон запрос.
    request_template = E_soap_env.Envelope(
        E_soap_env.Header(),
        E_soap_env.Body(
            E_ns.GetIncomingQueueStatisticsRequest(
                E_ns1.Timestamp()
            )
        )
    )

    method = RequestTypes.GetIncomingQueueStatistics

    def __init__(self, timestamp, *args, **kwargs):
        super(GetIncomingQueueStatisticsRequest, self).__init__(
            *args, **kwargs)

        self._timestamp = timestamp
        self._timestamp_tag = None

    @property
    def timestamp(self):
        u"""Объект, который содержит время.

        По этому времени будут фильтроваться события в очереди.
        """

        if self._timestamp_tag is None:
            self._timestamp_tag = self._request.find(
                ".//{{{0}}}Timestamp".format(ns1))

        return self._timestamp_tag

    def _set_data(self):
        u"""Установка параметров запроса.

        Выставляем время последней отправки.
        """
        super(GetIncomingQueueStatisticsRequest, self)._set_data()
        self.timestamp.text = self._timestamp

    def _sign(self):
        u"""Метод выполняет подпись элементов конверта."""
        super(GetIncomingQueueStatisticsRequest, self)._sign()

        caller_information_system_signature = self._signer.sign(self.timestamp)

        get_incoming_queue_statistics_request = self._request.find(
            ".//{{{0}}}GetIncomingQueueStatisticsRequest".format(ns))
        get_incoming_queue_statistics_request.append(
            caller_information_system_signature)


OpenSSLGetIncomingQueueStatisticsRequest = partial(
    GetIncomingQueueStatisticsRequest,
    signer=EPOVOpenSSLSigner()
)


class GetIncomingQueueStatisticsResponse(BaseResponse):
    u"""Ответ метода GetIncomingQueueStatisticsRequest.

    Метод используется для получения статистики очередей
    """

    @property
    def queue_statistics(self):
        u"""Свойство содержит словарь со статистикой очередей."""

        return dict((
            (_f.get('queueName'), _f.get('pendingMessageNumber')) for _f in self._response.findall(
            ".//{{{0}}}QueueStatistics".format(ns))))
