# -*- coding: utf-8 -*-

from functools import partial

from .base import BaseRequest
from .base import BaseResponse
from ..element_maker import E_soap_env
from ..element_maker import E_ns1
from ..element_maker import E_ns
from ..namespaces import ns
from ..namespaces import ns1
from ..security.sign import EPOVOpenSSLSigner
from .models import RequestTypes


class GetStatusRequest(BaseRequest):
    u"""Запрос метода GetStatus.

    Метод используется для опроса состояния очереди в системе СМЭВ3.
    """

    # Шаблон запрос.
    request_template = E_soap_env.Envelope(
        E_soap_env.Header(),
        E_soap_env.Body(
            E_ns.GetStatusRequest(
                E_ns1.Timestamp()
            )
        )
    )

    method = RequestTypes.GetStatus

    def __init__(self, timestamp, *args, **kwargs):
        super(GetStatusRequest, self).__init__(
            *args, **kwargs)

        self._timestamp = timestamp
        self._timestamp_tag = None

    @property
    def timestamp(self):
        u"""Объект, который содержит время.

        По этому времени будут фильтроваться события в очереди.
        """

        if self._timestamp_tag is None:
            self._timestamp_tag = self._request.find(
                ".//{{{0}}}Timestamp".format(ns1))

        return self._timestamp_tag

    def _set_data(self):
        u"""Установка параметров запроса.

        Выставляем время последней отправки.
        """
        super(GetStatusRequest, self)._set_data()
        self.timestamp.text = self._timestamp

    def _sign(self):
        u"""Метод выполняет подпись элементов конверта."""
        super(GetStatusRequest, self)._sign()

        caller_information_system_signature = self._signer.sign(self.timestamp)

        get_incoming_queue_statistics_request = self._request.find(
            ".//{{{0}}}GetStatusRequest".format(ns))
        get_incoming_queue_statistics_request.append(
            caller_information_system_signature)


OpenSSLGetStatusRequest = partial(
    GetStatusRequest,
    signer=EPOVOpenSSLSigner()
)


class GetStatusResponse(BaseResponse):
    u"""Ответ метода GetStatusResponse.

    Метод используется для получения статистики очередей
    """
