# -*- coding: utf-8 -*-

from datetime import datetime

from django.db import models

from django.utils.translation import ugettext as _


class RequestTypes(object):
    GetIncomingQueueStatistics = 'GetIncomingQueueStatistics'
    GetStatus = 'GetStatus'
    SendRequest = 'SendRequest'
    GetResponse = 'GetResponse'
    GetRequest = 'GetRequest'
    SendResponse = 'SendResponse'
    Ack = 'Ack'

    values = (
        (GetIncomingQueueStatistics, _('GetIncomingQueueStatistics')),
        (GetStatus, _('GetStatus')),
        (SendRequest, _('SendRequest')),
        (GetResponse, _('GetResponse')),
        (GetRequest, _('GetRequest')),
        (SendResponse, _('SendResponse')),
        (Ack, _('Ack')))


class AbstractRequests(models.Model):
    u"""Django модель для хранения информации о запросах в СМЭВ 3."""

    timestamp = models.DateTimeField(
        default=datetime.now,
        verbose_name=u"Дата/время запроса")

    original_message_id = models.CharField(
        max_length=256, null=True,
        blank=True,
        verbose_name=u"Идентификатор исходного запроса")

    message_id = models.CharField(
        max_length=256, null=True,
        blank=True,
        verbose_name=u"Идентификатор запроса")

    url = models.CharField(
        max_length=2048, null=False,
        blank=False,
        verbose_name=u"Адрес веб-сервиса")

    type = models.CharField(
        max_length=256, null=True,
        blank=True, choices=RequestTypes.values,
        verbose_name=u"Тип запроса")

    message = models.CharField(
        max_length=2048, null=True,
        blank=True,
        verbose_name=u"Вид сведений")

    method = models.CharField(
        max_length=2048, null=True,
        blank=True,
        verbose_name=u"Метод")

    request = models.TextField(
        verbose_name=u"Запрос")

    response = models.TextField(
        verbose_name=u"Ответ")

    error = models.TextField(
        null=True, blank=True,
        verbose_name=u"Возникшая ошибка")

    processed = models.BooleanField(
        default=False,
        verbose_name=u"Запрос обработан")

    class Meta:
        abstract = True

        verbose_name = u"Запрос веб-сервиса (СМЭВ 3)"
        verbose_name_plural = u"Журнал запросов веб-сервисов (СМЭВ 3)"

    def __unicode__(self):
        return self.type
