# coding: utf-8

from abc import ABCMeta
from abc import abstractmethod

from django.conf import settings
from django.utils.functional import cached_property
from educommon import ioc


class BaseExtensions(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_change(self, data, replace):
        u"""Возвращает объект-изменение."""
        pass

    def get_current_user(self, request):
        u"""Возвращает текущего пользователя."""
        return ioc.get_current_user(request)

    @cached_property
    def change_viewer_class(self):
        u"""Класс для отображения принятых изменений."""
        from .wrappers import AppliedChangeViewer
        return AppliedChangeViewer

    @cached_property
    def person_model_class(self):
        u"""Класс физ. лица."""
        from datatransfer.source.common.configuration import container
        return container.get('person_models', settings.SYSTEM_NAME)

    @abstractmethod
    def apply_changes(self, person_data_ids, request, replace):
        u"""Принимает все изменения для заданных id."""
        pass
